# -*- coding:utf-8 -*-
from Queue import Empty
from bisect import bisect_left,bisect_right,insort

#定义节点
class SNode:
    def __init__(self,key=None, score=float('-inf'),next=None):
        self.key   = key
        self.score = score

    def __lt__(self,other):
        return self.score < getattr(other,'score',other)

    def __gt__(self,other):#没定义__gt__的话会导致bisect_right出问题,即使已经定义了__lt__
        return self.score > getattr(other,'score',other)

#定义数组,用bisect维护顺序
class Slist(object):
    def __init__(self):
        self.key2node = {}
        self.card = 0
        self.orderlist = []

    def findpos(self, snode):
        curpos = bisect_left(self.orderlist,snode)
        while 1:
            if self.orderlist[curpos].key==snode.key:
                break
            curpos += 1
        return curpos

    def insert(self,key,score):
        if not isinstance(score,int):raise Exception('score must be integer')
        snode = self.key2node.get(key)
        if snode:
            if score == snode.score:
                return 0
            del self.orderlist[self.findpos(snode)]
            snode.score = score
        else:
            self.card += 1
            snode = SNode(key=key,score=score)
            self.key2node[key] = snode
        insort(self.orderlist, snode)
        return 1

    def delete(self,key):
        snode = self.key2node.get(key)
        if not snode:
            return 0
        self.card -= 1
        del self.orderlist[self.findpos(snode)]
        del self.key2node[key]
        del snode
        return 1

    def search(self,key):
        return self.key2node.get(key)

class SortedSet:
    def __init__(self):
        self.slist = Slist()

    def zadd(self, key, score):
        return self.slist.insert(key, score)

    def zrem(self, key):
        return self.slist.delete(key)

    def zrank(self, key):#score相同则按字典序
        snode = self.slist.key2node.get(key)
        if not snode:
            return None
        return self.slist.findpos(snode)

    def zrevrank(self, key):
        return self.zcard - 1 - self.zrank(key)

    def zscore(self, key):
        snode = self.slist.key2node.get(key)
        return getattr(snode,'score',None)

    def zcount(self, start, end):
        ol = self.slist.orderlist
        return bisect_left(ol,end+1) - bisect_right(ol,start-1)

    @property
    def zcard(self):
        return self.slist.card

    def zrange(self, start, end, withscores=False):#score相同则按字典序
        nodes = self.slist.orderlist[start: end+1]
        if not nodes:return []
        if withscores:
            return [(x.key, x.score) for x in nodes]
        else:
            return [x.key for x in nodes]

    def zrevrange(self, start, end, withscores=False):
        card = self.zcard
        if end<0:
            end = end + card
        if start<0:
            start = start + card
        nodes = self.slist.orderlist[max(card-end-1, 0): max(card-start, 0)][::-1]
        if not nodes:return []
        if withscores:
            return [(x.key, x.score) for x in nodes]
        else:
            return [x.key for x in nodes]

    def zrangebyscore(self, start, end, withscores=False):
        ol = self.slist.orderlist
        nodes = ol[bisect_left(ol, start):bisect_right(ol, end)]
        if not nodes:return []
        if withscores:
            return [(x.key, x.score) for x in nodes]
        else:
            return [x.key for x in nodes]

    def zpop(self):
        data = self.zrange(0, 0)
        if data:
            data = data[0]
            self.zrem(data)
            return data
        raise Empty

    def zrevrangebyscore(self, end, start, withscores=False):
        return self.zrangebyscore(start, end, withscores)[::-1]

    def zincrby(self, key):
        snode = self.slist.key2node.get(key)
        if not snode:
            return self.zadd(key, 1)
        score = snode.score
        self.zrem(key)
        return self.zadd(key, score+1)

    def __len__(self):
        return self.zcard



# -*- coding:utf-8 -*-
"""redis_server"""
import os
import time
import errno
import socket
import select
import fnmatch
import traceback

from argparse import ArgumentParser
from threading import Thread

from multi_thread_closing import MultiThreadClosing

from errors import MethodNotExist, ClientClosed
from utils import stream_wrapper
from default_data_types import *


class CustomRedis(MultiThreadClosing):
    name = "redis_server"
    default = {"str": StrStore, "hash": HashStore, "set": SetStore, "zset": ZsetStore, "list": ListStore}

    def __init__(self, host, port):
        MultiThreadClosing.__init__(self)
        self.host = host
        self.port = port
        self.data_type = {}
        self.datas = {}
        self.expire_keys = {}
        self.open()
        self.data_type.update(self.default)

    def install(self, **kwds):
        self.data_type.update(kwds)

    def setup(self):
        if os.path.exists("redis_data.db"):
            lines = open("redis_data.db").read().split("fdfsafafdsfsfdsfafdff")
            if lines:
                for line in lines:
                    self.load(line)

    def load(self, line):
        try:
            if line:
                key, expire_time, val = line.split("    ")
                val = pickle.loads(val)
                if len(self.data_type) < 2:
                    cls = self.data_type.values()[0].loads(val)
                else:
                    cls = reduce(lambda x, y:
                                 (x.loads(val) if hasattr(x, "loads") else None) or
                                 y.loads(val),
                                 self.data_type.values())
                if cls:
                    self.datas[key] = cls(self.logger, val)
                    if expire_time:
                        self.expire_keys[key] = int(expire_time)
        except Exception:
            self.logger.error(traceback.format_exc())

    def __getattr__(self, item):
        for k, v in self.data_type.items():
            attr = getattr(v, item, None)
            if attr:
                return getattr(v.from_redis(self), item)
        raise MethodNotExist

    def start(self):
        self.setup()
        daemaon_thread = Thread(target=self.poll)
        daemaon_thread.setDaemon(True)
        daemaon_thread.start()
        self.listen_request(self.host, self.port)

    def poll(self):
        while True:
            for key in self.expire_keys.keys():
                if self.expire_keys[key] < time.time():
                    del self.datas[key], self.expire_keys[key]
            time.sleep(1)

    def listen_request(self, host, port):
        """监听函数"""
        self.logger.debug("listen  to %s:%s"%(host, port))
        server = socket.socket()
        server.bind((host, port))
        server.listen(0)
        r_lst = {server: None}
        w_lst = {}
        # 若执行过程中出现异常r_lst中的server也被清掉，程序退出
        try:
            while self.alive and r_lst:
                readable, writable, _ = select.select(r_lst.keys(), w_lst.keys(), [], 0.1)
                for r in readable:
                    self.recv(r, w_lst, r_lst, server)
                for w in writable:
                    self.send(w, w_lst, r_lst, None)
                    # time.sleep(1)
        except select.error, e:
            if e.args[0] != 4:
                raise
        self.logger.info("exit...")

    def stop(self, *args):
        self.alive = False
        stream = open("redis_data.db", "w")
        self.logger.info("persist datas...")
        for key, val in self.datas.items():
            stream.write(key)
            stream.write('    %s' % self.expire_keys.get(key, ""))
            stream.write('    ')
            val.persist(stream)
        stream.close()

    @stream_wrapper
    def send(self, w, w_lst, r_lst, server):
        # item会被保存在w_lst相应的val中
        item = w_lst[w]
        self.logger.debug("start to send item %s to %s:%s" % ((item,) + r_lst[w][:2]))
        w.send(item)

    @stream_wrapper
    def recv(self, r, w_lst, r_lst, server):
        if r is server:
            client, adr = r.accept()
            self.logger.debug("get connection from %s:%s" % adr)
            # 将新收到的socket设置为非阻塞， 并将其保存在r_lst中
            client.setblocking(0)
            r_lst[client] = adr
        else:
            self.logger.debug("start to recv data from %s:%s" % r_lst[r][:2])
            received = self._recv(r)
            if received:
                cmd, data, keep = received.split("#-*-#")
                key, val = data.split("<->")
                # 根据指令生成item返回结果
                try:
                    item = (getattr(self.datas.get(key, None), cmd, None) or getattr(self, cmd))(key, val, self)
                except MethodNotExist:
                    item = "404#-*-#Method Not Found#-*-#\r\n\r\n"
                # 将socket保存在w_lst中，并将keep-alive 标志保存在其val中
                w_lst[r] = item
                r_lst[r] = r_lst[r][:2] + (keep,)
            else:
                raise ClientClosed("closed")

    def _recv(self, r):
        msg = ""
        try:
            buf = r.recv(1024)
            while buf:
                msg += buf
                buf = r.recv(1024)
        except Exception, e:
            # 非阻塞异常直接返回
            if e.args[0] != errno.EAGAIN:
                # traceback.print_exc()
                pass
        self.logger.info("received massage is %s" % (msg or None))
        return msg

    def keys(self, k, v, instance):
        return "%s#-*-#%s#-*-#%s\r\n\r\n" % ("200", "success",
                                     json.dumps(filter(lambda x: fnmatch.fnmatch(x, k), self.datas.keys())))

    def expire(self, k, v, instance):
        self.expire_keys[k] = int(time.time() + int(v))
        return "200#-*-#success#-*-#\r\n\r\n"

    def type(self, k, v, instance):
        data = self.datas[k]
        return "200#-*-#success#-*-#%s\r\n\r\n"%data.__class__.__name__[:-5].lower()

    def ttl(self, k, v, instance):
        expire = self.expire_keys.get(k)
        if expire:
            expire = int(expire - time.time())
        else:
            expire = -1
        return "200#-*-#success#-*-#%d\r\n\r\n" % expire

    def delete(self, k, v, instance):
        try:
            del self.datas[k]
        except KeyError:
            pass
        return "200#-*-#success#-*-#\r\n\r\n"

    def flushall(self, k, v, instance):
        self.datas = {}
        return "200#-*-#success#-*-#\r\n\r\n"

    @classmethod
    def parse_args(cls):
        parser = ArgumentParser()
        parser.add_argument("--host", dest="host", help="host", default="127.0.0.1")
        parser.add_argument("-p", "--port", type=int, dest="port", help="port", default=7777)
        return cls(**vars(parser.parse_args()))


def start_server():
    cr = CustomRedis.parse_args()
    cr.set_logger()
    cr.start()


if __name__ == "__main__":
    start_server()



# -*- coding:utf-8 -*-
import pickle
import random
import json

from zset import SortedSet
from Queue import Queue, Empty
from utils import cmd_wrapper


class DataStore(object):

    data_type = None

    def __init__(self, logger, data=None):
        self.logger = logger
        self.data = data or self.data_type()

    @classmethod
    def from_redis(cls, redis):
        return cls(redis.logger)

    def format_response(self, code, info, data):
        if data is None:
            data = ""
        return "%s#-*-#%s#-*-#%s\r\n\r\n" % (code, info, data)

    def persist(self, stream):
        stream.write(pickle.dumps(self.data))
        stream.write("fdfsafafdsfsfdsfafdff")

    def _parses(self, v):
        return json.loads(v)

    @classmethod
    def loads(cls, val):
        if isinstance(val, cls.data_type):
            return cls


class ZsetStore(DataStore):
    data_type = SortedSet

    @cmd_wrapper
    def zadd(self, k, v, instance):
        k, v = self._parses(v).items()[0]
        self.data.zadd(v, int(k))

    @cmd_wrapper
    def zpop(self, k, v, instance):
        return self.data.zpop()

    @cmd_wrapper
    def zcard(self, k, v, instance):
        return self.data.zcard


class QueueStore(DataStore):
    data_type = Queue

    @cmd_wrapper
    def pop(self, k, v, instance):
        return self.data.get_nowait()

    @cmd_wrapper
    def push(self, k, v, instance):
        self.data.put(v)


class ListStore(DataStore):
    data_type = list

    @cmd_wrapper
    def lpop(self, k, v, instance):
        if self.data:
            return self.data.pop(0)
        else:
            raise Empty

    @cmd_wrapper
    def rpush(self, k, v, instance):
        self.data.append(v)

    @cmd_wrapper
    def llen(self, k, v, instance):
        return len(self.data)


class StrStore(DataStore):
    data_type = str

    @cmd_wrapper
    def add(self, k, v, instance):
        self.data += v

    @cmd_wrapper
    def slice(self, k, v, instance):
        return eval("self.data[%s]" % v)

    @cmd_wrapper
    def set(self, k, v, instance):
        self.data = v

    @cmd_wrapper
    def get(self, k, v, instance):
        return self.data


class SetStore(DataStore):
    data_type = set

    @cmd_wrapper
    def sadd(self, k, v, instance):
        self.data.add(v)

    @cmd_wrapper
    def smembers(self, k, v, instance):
        return len(self.data)

    @cmd_wrapper
    def remove(self, k, v, instance):
        self.data.remove(v)

    @cmd_wrapper
    def rchoice(self, k, v, instance):
        return random.choice(list(self.data))


class HashStore(DataStore):
    data_type = dict

    @cmd_wrapper
    def hset(self, k, v, instance):
        self.data.update(self._parses(v))

    @cmd_wrapper
    def hget(self, k, v, instance):
        return self.data[v]

    @cmd_wrapper
    def hmset(self, k, v, instance):
        k_vs = self._parses(v)
        self.data.update(dict(k_vs))

    @cmd_wrapper
    def hmget(self, k, v, instance):
        ks = self._parses(v)
        return json.dumps(dict(filter(lambda x: x[0] in ks, self.data.items())))

    @cmd_wrapper
    def hgetall(self, k, v, instance):
        return json.dumps(self.data)

    @cmd_wrapper
    def hincrby(self, k, v, instance):
        k_vs = self._parses(v)
        k = k_vs.keys()[0]
        v = k_vs[k]
        self.data[k] = int(self.data.get(k, 0)) + int(v)



# -*- coding:utf-8 -*-
"""redis_client"""
import json
import time
import errno
import argparse

from socket import socket

from functions import CMD_DICT
from errors import RedisArgumentError, RedisError
from utils import SafeList, func_name_wrapper, handle_safely, default_recv, default_send

FORMAT = "%s#-*-#%s#-*-#1"


class Redis(object):

    def __init__(self, host="localhost", port=6379, timeout=30):
        self.host = host
        self.port = port
        self.timeout = timeout
        self.redis_conn = None
        self.setup()

    def setup(self):
        if self.redis_conn:
            try:
                self.redis_conn.close()
            except:
                pass
        self.redis_conn = socket()
        self.redis_conn.connect((self.host, self.port))
        self.redis_conn.settimeout(self.timeout)

    def __getattr__(self, name):
        return func_name_wrapper(name)(self._execute_cmd)

    def _execute_cmd(self, *args, **kwargs):
        args = SafeList(args)
        func_name = kwargs["_name"]
        properties = CMD_DICT[func_name]
        default_value = properties.get("default", [])
        try:
            arguments = []
            for arg in properties["args"]:
                kwarg = kwargs.pop(arg, None)
                if kwarg is None:
                    kwarg = args.pop(0)
                    if kwarg is None:
                        kwarg = default_value.pop(0)
                arguments.append(kwarg)
        except IndexError:
            raise RedisArgumentError("haven't got enough arguments")
        if args:
            arguments += args
        return self._parse_result(FORMAT % (func_name, properties.get("send", default_send)(*arguments)), properties)

    def _parse_result(self, buf, properties={}):
        count = 0
        result = ""
        try:
            self.redis_conn.send(buf)
        except Exception as e:
            if e.args[0] == errno.EPIPE and count < 3:
                self.setup()
                count += 1
                time.sleep(1)
            else:
                raise
        while True:
            recv = self.redis_conn.recv(1024000)
            if recv:
                result += recv
            if not recv or recv.endswith("\r\n\r\n"):
                break
        a = result.split("#-*-#")
        code, info, data = a
        data = data[:-4]
        if code == "200":
            return handle_safely(properties.get("recv", default_recv))(data)
        elif code == "502":
            return properties.get("result", data)
        else:
            raise RedisError("%s:%s, %s"%(code, info, data))

    def keys(self, pattern="*"):
        return self._parse_result(FORMAT%("keys", "%s<->"%pattern), {"recv":json.loads})

    def type(self, key):
        return self._parse_result(FORMAT % ("type", "%s<->" % key))

    def delete(self, key):
        return self._parse_result(FORMAT % ("delete", "%s<->" % key))

    def expire(self, key, seconds):
        return self._parse_result(FORMAT % ("expire", "%s<->%s" % (key, seconds)))

    def ttl(self, key):
        return self._parse_result(FORMAT % ("ttl", "%s<->" % key))

    def flushall(self):
        return self._parse_result(FORMAT % ("flushall", "<->"))


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--host", dest="host", default="127.0.0.1")
    parser.add_argument("-p", "--port", dest="port", type=int, default=6379)
    parser.add_argument("-c", "--cmd", dest="cmd", required=True)
    parser.add_argument("args", nargs="*", default=[])
    parser.add_argument("-k", "--key", dest="key")
    parser.add_argument("-j", "--json", dest="json", action="store_true")
    parser.add_argument("--keep-alive", dest="keep_alive", action="store_true")
    return parser.parse_args()


def start_client():
    args = parse_args()
    r = Redis(args.host, args.port)
    keys = [args.key] if args.key else []
    if not args.keep_alive:
        FORMAT.replace("1", "0")
    if args.json:
        mapping = [json.loads(args.args[0])]
        result = getattr(r, args.cmd)(*(keys + mapping))
    else:
        result = getattr(r, args.cmd)(*(keys + args.args))
    if result != None:
        print result


if __name__ == "__main__":
    start_client()




# -*- coding:utf-8 -*-
import traceback

from Queue import Empty
from functools import wraps

from errors import ClientClosed


def stream_wrapper(func):
    """
    处理异常的装饰器
    :param func: 装饰的函数
    :return: 装饰好的函数
    """

    @wraps(func)
    def wrapper(*args):

        self, stream, w_lst, r_lst, server = args
        is_error = None
        is_closed = None
        try:
            return func(*args)
        except ClientClosed:
            is_closed = True
            is_error = True
        except Exception:
            self.logger.info(traceback.format_exc())
            is_error = True
        finally:
            if is_error or server is None:
                # 只有send的时候，server才会为空
                # 也就是说出了异常，或者send时，才会做下面的操作
                if stream in w_lst:
                    del w_lst[stream]
                # 如果每个socket属性元组中keep-alive 不为空，则不关闭stream
                if (stream in r_lst and (len(r_lst[stream]) != 3 or not int(r_lst[stream][2]))) or is_closed:
                    del r_lst[stream]
                    try:
                        stream.close()
                    except:
                        pass

    return wrapper


def cmd_wrapper(func):
    @wraps(func)
    def wrapper(*args):

        self, k, v, instance = args
        try:
            self.key = k
            instance.datas[k] = self
            self.logger.info("process in method %s" % func.__name__)
            return self.format_response(200, "success", func(*args))
        except (Empty, KeyError):
            if not self.data:
                del instance.datas[k]
            self.logger.error(traceback.format_exc())
            return self.format_response(502, "Empty", "")
        except Exception as e:
            self.logger.error(traceback.format_exc())
            return self.format_response(503, e.__class__.__name__.lower(), v)

    return wrapper


# -*- coding:utf-8 -*-
import traceback



class SafeList(list):
    def pop(self, index=-1):
        try:
            return super(SafeList, self).pop(index)
        except IndexError:
            return None


def default_send(*args):
    if len(args) == 1:
        return "%s<->"%tuple(args)
    else:
        return "%s<->%s"%tuple(args)


def default_recv(x):
    return x


def func_name_wrapper(name):
    def wrapper(func):
        def inner_wrapper(*args, **kwargs):
            kwargs["_name"] = name
            return func(*args, **kwargs)
        return inner_wrapper
    return wrapper


def handle_safely(func):
    def wrapper(data):
        try:
            return func(data)
        except Exception:
            traceback.print_exc()
            return data
    return wrapper




# -*- coding:utf-8 -*-
import codecs
import os
try:
    from setuptools import setup, find_packages
except:
    from distutils.core import setup


VERSION = '1.0.8'

AUTHOR = "cn"

AUTHOR_EMAIL = "308299269@qq.com"

URL = "https://www.github.com/ShichaoMa/custom_redis"

NAME = "custom-redis"

DESCRIPTION = "python实现简单redis，实现redis基本功能以及可插拔数据结构。 "


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()

LONG_DESCRIPTION = read("README.rst")

KEYWORDS = "custom redis"

LICENSE = "MIT"

PACKAGES = ["custom_redis", "custom_redis.client", "custom_redis.server"]

setup(
    name = NAME,
    version = VERSION,
    description = DESCRIPTION,
    long_description = LONG_DESCRIPTION,
    classifiers = [
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
    ],
    entry_points={
        'console_scripts': [
            'custom-redis-server = custom_redis:start_server',
            'custom-redis-client = custom_redis:start_client',
        ],
    },
    keywords = KEYWORDS,
    author = AUTHOR,
    author_email = AUTHOR_EMAIL,
    url = URL,
    license = LICENSE,
    packages = PACKAGES,
    install_requires=["multi-thread-closing"],
    include_package_data=True,
    zip_safe=True,
)


# -*- coding:utf-8 -*-
import json


CMD_DICT = {
    "set": {
        "args": ["name", "value"],
        "send":lambda *args:"%s<->%s"%tuple(args),
    },
    "get": {
        "args": ["name",],
    },
    "hget":{
        "args":["name", "key"],
    },
    "hset":{
        "args": ["name", "key", "value"],
        "send":lambda *args:"%s<->%s"%(args[0], json.dumps(dict([args[1:]]))),
    },
    "hmset": {
        "args": ["name", "mapping"],
        "send":lambda *args:"%s<->%s"%(args[0], json.dumps(args[1])),
    },
    "hmget": {
        "args": ["name", "key"],
        "send": lambda *args:"%s<->%s"%(args[0], json.dumps(args[1:])),
        "recv": lambda data:json.loads(data),
    },
    "hgetall": {
        "args": ["name"],
        "send": lambda *args: "%s<->" % args[0],
        "recv": lambda data: json.loads(data),
    },
    "hincrby": {
        "args": ["name", "key", "value"],
        "send": lambda *args:"%s<->%s"%(args[0], json.dumps(dict([args[1:]]))),
        "default":[1]
    },
    "pop": {
        "args": ["name"],
        "recv": lambda data: json.loads(data),
    },
    "push": {
        "args": ["name", "value"],
        "send": lambda *args: "%s<->%s" % (args[0], json.dumps(args[1])),
    },
    "zadd": {
        "args": ["name", "key", "value"],
        "send": lambda *args: "%s<->%s" % (args[0], json.dumps(dict([args[1:]]))),
    },
    "zpop": {
        "args": ["name"],
        "send": lambda *args: "%s<->" % args[0],
    },
    "zcard": {
        "args": ["name"],
        "send": lambda *args: "%s<->" % args[0],
        "recv": lambda x:int(x),
        "result": 0
    },
    "lpop": {
        "args": ["name"],
        "send": lambda *args: "%s<->" % args[0],
    },
    "rpush": {
        "args": ["name", "value"],
        "send": lambda *args: "%s<->%s" % (args[0], args[1]),
    },
    "llen": {
        "args": ["name"],
        "send": lambda *args: "%s<->" % args[0],
        "recv": lambda x: int(x),
        "result": 0
    },
}
